# blink2png-bridge  Copyright (C) 2020 Star Inc.
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import subprocess


class Blink2pngBridge:
    config = {
        "xvfb-run_path": "xvfb-run",
        "execute_path": "blink2png",
        "wait": "0",
        "timeout": "10",
        "width": "1024",
        "height": "768",
        "save_path": os.getcwd()
    }

    def __init__(self, custom_config=None):
        """
        Configure custom config if existed.
        :param custom_config: dict
        """
        if custom_config is None:
            custom_config = {}
        for key in custom_config:
            if key in self.config:
                self.config[key] = str(custom_config.get(key))

    def set_wait(self, seconds):
        """
        Set time to wait after loading before the screenshot is taken.
        :param seconds: integer
        :return:
        """
        self.config["wait"] = str(seconds)

    def set_timeout(self, seconds):
        """
        Set time before the request will be canceled.
        :param seconds: integer
        :return:
        """
        self.config["timeout"] = str(seconds)

    def set_save_path(self, path):
        """
        Set save path of snapshots.
        :param path: string (ex. /tmp/snapshot/)
        :return:
        """
        self.config["save_path"] = os.path.join(path)

    def set_window_size(self, width, height):
        """
        Set picture size of snapshots.
        :param width: integer
        :param height: integer
        :return:
        """
        self.config["width"] = str(width)
        self.config["height"] = str(height)

    def save_screenshot(self, url, filename="capture.png"):
        """
        Take snapshot of web page with URL.
        :param url: string
        :param filename: string
        :return:
        """
        xvfb_run = self.config.get("xvfb-run_path")
        execute_path = self.config.get("execute_path")

        wait = self.config.get("wait")
        timeout = self.config.get("timeout")

        width = self.config.get("width")
        height = self.config.get("height")

        save_path = self.config.get("save_path")

        subprocess.Popen([
            xvfb_run,
            execute_path,
            "-w", wait,
            "-t", timeout,
            "-g", width, height,
            "-o", filename,
            url
        ], cwd=save_path, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
